"""
Acoustics processing (:mod:`acous.acoustics`)
=======================================

Wavelets
--------


Sound Pressure Level (SPL)
--------------------------
   from_pressure_to_spl - Calculate sound pressure level from sound pressure.
   from_spl_to_pressure - Calculate sound pressure from sound pressure level.
   from_spl_to_oaspl - Calculate overall sound pressure level from sound pressure level.

"""
from .spl import *