"""
Signal processing (:mod:`acous.signal`)
=======================================

Wavelets
--------


Spectral analysis
-----------------
   periodogram - Compute a (modified) periodogram.
   welch - Compute a periodogram using Welch's method.

"""
from . import windows
from .spectral import *