"""
Window functions (:mod:`scipy.signal.windows`)
==============================================

The suite of window functions for filtering and spectral estimation.

Window functions
----------------
   hanning - Hann window

"""
from .windows import *