"""
Plot setup (:mod:`acous.plot`)
==============================

Plot setup functions
--------------------
   plot_setup - Define font size of generated pictures.

Helper routines
---------------
   read_data - Read data from a file

"""
from .plot_setup import *